/*
 * Freescale STMP37XX/STMP378X SoC register access interfaces
 *
 * The SoC registers may be accessed via:
 *
 * - single 32 bit address, or
 * - four 32 bit addresses - general purpose, set, clear and toggle bits
 *
 * Multiple IP blocks (e.g. SSP, UART) provide identical register sets per
 * each module
 *
 * Embedded Alley Solutions, Inc <source@embeddedalley.com>
 *
 * Copyright 2008-2009 Freescale Semiconductor, Inc. All Rights Reserved.
 * Copyright 2008 Embedded Alley Solutions, Inc All Rights Reserved.
 */

/*
 * The code contained herein is licensed under the GNU General Public
 * License. You may obtain a copy of the GNU General Public License
 * Version 2 or later at the following locations:
 *
 * http://www.opensource.org/licenses/gpl-license.html
 * http://www.gnu.org/copyleft/gpl.html
 */
#ifndef __ASM_PLAT_STMP3XXX_REGS_H
#define __ASM_PLAT_STMP3XXX_REGS_H


#define REGS_BASE 0x80000000

#define HW_STMP3xxx_SET		0x04
#define HW_STMP3xxx_CLR		0x08
#define HW_STMP3xxx_TOG		0x0c

#ifndef __ASSEMBLER__

#define HW_REGISTER_WO(id, base, offset) HW_REGISTER(id, base, offset)

#define HW_REGISTER_RO(id, base, offset) HW_REGISTER(id, base, offset)

#define HW_REGISTER(id, base, offset)	 \
	static inline unsigned int id##_RD() { \
		return	*(volatile int *)((base)+(offset));\
	} \
	static inline void id##_WR(unsigned int v) {\
		*(volatile int *)((base)+(offset)) = v;\
	} \
	static inline void id##_SET(unsigned int v) {\
		*(volatile int *)((base)+(offset) + 0x4) = v;\
	} \
	static inline void id##_CLR(unsigned int v) {\
		*(volatile int *)((base)+(offset) + 0x8) = v;\
	} \
	static inline void id##_TOG(unsigned int v) {\
		*(volatile int *)((base)+(offset) + 0xc) = v;\
	} \

#define HW_REGISTER_0(id, base, offset)	 HW_REGISTER(id, base, offset)

#define HW_REGISTER_INDEXED(id, base, offset, step)

#define HW_REGISTER_RO_INDEXED(id, base, offset, step)	\
		HW_REGISTER_RO_INDEXED(id, base, offset, step)
#define HW_REGISTER_0_INDEXED(id, base, offset, step)	\
		HW_REGISTER_RO_INDEXED(id, base, offset, step)

#else /* __ASSEMBLER__ */

#define HW_REGISTER_FUNCS(id, base, offset, regset, rd, wr)
#define HW_REGISTER_FUNCS_INDEXED(id, base, offset, regset, rd, wr, step)
#define HW_REGISTER_WO(id, base, offset)
#define HW_REGISTER_RO(id, base, offset)
#define HW_REGISTER(id, base, offset)
#define HW_REGISTER_0(id, base, offset)
#define HW_REGISTER_INDEXED(id, base, offset, step)
#define HW_REGISTER_RO_INDEXED(id, base, offset, step)
#define HW_REGISTER_0_INDEXED(id, base, offset, step)
#endif /* __ASSEMBLER__ */

#endif /* __ASM_PLAT_STMP3XXX_REGS_H */
